/*
<:copyright-gpl 
 Copyright 2002 Broadcom Corp. All Rights Reserved. 
 
 This program is free software; you can distribute it and/or modify it 
 under the terms of the GNU General Public License (Version 2) as 
 published by the Free Software Foundation. 
 
 This program is distributed in the hope it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
 for more details. 
 
 You should have received a copy of the GNU General Public License along 
 with this program; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place - Suite 330, Boston MA 02111-1307, USA. 
:>

 Copyright (C) 2000 Geert Uytterhoeven <geert@sonycom.com>
 Sony Software Development Center Europe (SDCE), Brussels
*/

#include <linux/config.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/console.h>
#include <linux/sched.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/module.h>

#include <asm/addrspace.h>
#include <asm/irq.h>
#include <asm/reboot.h>
#include <asm/gdb-stub.h>
#include <asm/mc146818rtc.h> 

#define stUart ((volatile Uart * const) UART_BASE)

#ifdef CONFIG_REMOTE_DEBUG
extern void rs_kgdb_hook(int);
extern void breakpoint(void);
#endif

#if defined(CONFIG_SERIAL_CONSOLE)
extern void console_setup(char *);
#endif

/* These routines are found in the arch/mips/kernel/irq.c 
 * connecting these externs to our hard interrupts makes the whole
 * Linux interrupt architecture work.
 * Also note the 8259 is the PIC on the ALi chip. TDT
 */
extern void __init i8259_init(void);   

#include <linux/mc146818rtc.h>
#include <linux/timex.h>

static unsigned char brcm_rtc_read_data(unsigned long addr)
{
    // Toggle the RTC Update-In-Progress flag so it looks like we have
    // a working RTC (at least for now).

    static int iUpdateFlag = RTC_UIP;
    unsigned char uReturn;

    if (addr == RTC_FREQ_SELECT) {
        uReturn = iUpdateFlag;
        iUpdateFlag ^= RTC_UIP;
    } else {
        uReturn = 0;
    }

    return uReturn;
}

static void 
brcm_rtc_write_data(unsigned char data, unsigned long addr)
{

}

static int 
brcm_rtc_bcd_mode(void)
{
    return 0;
}

struct rtc_ops brcm_rtc_ops = {
        &brcm_rtc_read_data,
        &brcm_rtc_write_data,
        &brcm_rtc_bcd_mode
};

void (*board_time_init)(struct irqaction *irq);

unsigned int cputickcnt = 0;
unsigned int sysClkTicksPerSecond = 100; /* every 10 ms */
#define CPU_CLOCK_RATE                  81000000
#define PERIOD_INC(x)                   (CPU_CLOCK_RATE/1000*x)
#define TICKS_TO_MS(x)                  ((x)/(CPU_CLOCK_RATE/1000))
#define CPU_PERIOD  ((unsigned long) CPU_CLOCK_RATE / 1000000)
void MipsTimerInit(unsigned long cnt);
void MipsTimerIntrEnable(void);

#if 0 /* These functions are not currently being used. */
static void (*back_to_prom)(void) = (void (*)(void))0xbfc00000;
static void brcm_machine_restart(char *command)
{
    /* CPU cold reset */
    /* Call the PROM */
    back_to_prom();
}

static void brcm_machine_halt(void)
{
    printk("BCM93352 halted.\n");
    do {} while (1);
}

static void brcm_machine_power_off(void)
{
    printk(KERN_INFO "SETUP: BCM93352 halted. Please turn off the power.\n");
    do {} while (1);
}
#endif

/***************************************************************************
 * C++ New and delete operator functions
 ***************************************************************************/

/* void *operator new(unsigned int sz) */
void *_Znwj(unsigned int sz)
{
    return( kmalloc(sz, GFP_KERNEL) );
}

/* void *operator new[](unsigned int sz)*/
void *_Znaj(unsigned int sz)
{
    return( kmalloc(sz, GFP_KERNEL) );
}

/* placement new operator */
/* void *operator new (unsigned int size, void *ptr) */
void *ZnwjPv(unsigned int size, void *ptr)
{
    return ptr;
}

/* void operator delete(void *m) */
void _ZdlPv(void *m)
{
    kfree(m);
}

/* void operator delete[](void *m) */
void _ZdaPv(void *m)
{
    kfree(m);
}

EXPORT_SYMBOL(_Znwj);
EXPORT_SYMBOL(_Znaj);
EXPORT_SYMBOL(ZnwjPv);
EXPORT_SYMBOL(_ZdlPv);
EXPORT_SYMBOL(_ZdaPv);

