/*
<:copyright-gpl 
 Copyright 2002 Broadcom Corp. All Rights Reserved. 
 
 This program is free software; you can distribute it and/or modify it 
 under the terms of the GNU General Public License (Version 2) as 
 published by the Free Software Foundation. 
 
 This program is distributed in the hope it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
 for more details. 
 
 You should have received a copy of the GNU General Public License along 
 with this program; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place - Suite 330, Boston MA 02111-1307, USA. 
:>
*/

#ifndef __BCM6345_MAP_H
#define __BCM6345_MAP_H

#if __cplusplus
extern "C" {
#endif

#include "bcmtypes.h"
#include "6345_intr.h"

typedef struct IntControl {
  uint32        RevID;
#define CHIPID          0x634507a0
  uint16        testControl;
  uint16        blkEnables;
#define UART_CLK_EN     0x0008

  uint32        pll_control;
#define SOFT_RESET	0x00000001

  uint32        IrqMask;
#define DMACH18         0x80000000
#define DMACH17         0x40000000
#define DMACH16         0x20000000
#define DMACH15         0x10000000
#define DMACH14         0x08000000
#define DMACH13         0x04000000
#define DMACH12         0x02000000
#define DMACH11         0x01000000
#define DMACH10         0x00800000
#define DMACH09         0x00400000
#define DMACH08         0x00200000
#define DMACH07         0x00100000
#define DMACH06         0x00080000
#define DMACH05         0x00040000
#define DMACH04         0x00020000
#define DMACH03         0x00010000
#define DMACH02         0x00008000
#define DMACH01         0x00004000

#define EPHYIRQ         0x00001000
#define EMACIRQ         0x00000100
#define USBIRQ          0x00000020
#define ATMIRQ          0x00000010
#define ADSLIRQ         0x00000008
#define UARTIRQ         0x00000004
#define TIMRIRQ         0x00000001

  uint32        IrqStatus;
  byte          unused3;

  byte          extLevelEdgeInsense;

  byte          extIrqMskandDetClr;

  byte          extIrqStatEdgeConfig;
} IntControl;

#define INTC_BASE     0xfffe0000
#define INTC ((volatile IntControl * const) INTC_BASE)

typedef struct UartChannel {
  byte          unused0;
  byte          control;
#define BRGEN           0x80    /* Control register bit defs */
#define TXEN            0x40
#define RXEN            0x20
#define TXPARITYEN      0x08
#define TXPARITYEVEN    0x04
#define RXPARITYEN      0x02
#define RXPARITYEVEN    0x01
  byte          config;
#define BITS5SYM        0x00
#define BITS6SYM        0x10
#define BITS7SYM        0x20
#define BITS8SYM        0x30
#define XMITBREAK       0x40
#define ONESTOP         0x07
#define TWOSTOP         0x0f

  byte          fifoctl;
#define RSTTXFIFOS      0x80
#define RSTRXFIFOS      0x40
  uint32        baudword;

  byte          txf_levl;
  byte          rxf_levl;
  byte          fifocfg;
  byte          prog_out;

  byte          unused1;
  byte          DeltaIPEdgeNoSense;
  byte          DeltaIPConfig_Mask;
  byte          DeltaIP_SyncIP;
  uint16        intMask;
  uint16        intStatus;
#define TXUNDERR        0x0002
#define TXOVFERR        0x0004
#define TXFIFOEMT       0x0020
#define RXOVFERR        0x0080
#define RXFIFONE        0x0800
#define RXFRAMERR       0x1000
#define RXPARERR        0x2000
#define RXBRK           0x4000

  uint16        unused2;
  uint16        Data;
  uint32		unused3;
  uint32		unused4;
} Uart;

#define UART_BASE     0xfffe0300
#define UART ((volatile Uart * const) UART_BASE)

typedef struct GpioControl {
  uint16        unused0;
  byte          unused1;
  byte          TBusSel;

  uint16        unused2;
  uint16        GPIODir;
  byte          unused3;
  byte          Leds;
  uint16        GPIOio;

  uint32        UartCtl;
} GpioControl;

#define GPIO_BASE     0xfffe0400
#define GPIO ((volatile GpioControl * const) GPIO_BASE)

typedef enum {
    LED_1A = 1,
    LED_2B,
    LED_3C,
    LED_4D
}LEDDEFS;

#define LEDON                   1
#define LEDOFF                  0

#if __cplusplus
}
#endif

#endif

