/*****************************************************************************
//
//  Copyright (c) 2000-2001  Broadcom Corporation
//  All Rights Reserved
//  No portions of this material may be reproduced in any form without the
//  written permission of:
//          Broadcom Corporation
//          16215 Alton Parkway
//          Irvine, California 92619
//  All information contained in this document is Broadcom Corporation
//  company private, proprietary, and trade secret.
//
******************************************************************************
//
//  Filename:       syscall.c
//  Author:         Peter T. Tran
//  Creation Date:  12/26/01
//
******************************************************************************
//  Description:
//      It provides system call functions for Linux.
//
*****************************************************************************/

/********************** Include Files ***************************************/

#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <pwd.h>
#include <crypt.h>
#include <time.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <net/if.h>
#include <net/route.h>
#include <string.h>
#include <syslog.h>

#include "bcmtypes.h"
#include "bcmadsl.h"
#include "syscall.h"
#include "board_api.h"
#include "ifcdefs.h"
#include "psiapi.h"
#include "sysdiag.h"
#include "clidefs.h"
#include "cliapi.h"
#include "adslctlapi.h"

#include <netdb.h>
#include <cgimain.h>
#include <icmapi.h>

#ifdef BUILD_STATIC
#include "busybox.h"
#endif

extern char **environ;
extern char *getIfName(void);

//If we don't link with busybox, we need this function
#ifndef BUILD_STATIC
#define READ_BUF_SIZE        128
static void remove_delimitor( char *s)
{
    char *p1, *p2;

    p1 = p2 = s;
    while ( *p1 != '\0' || *(p1+1) != '\0') {
        if (*p1 != '\0') {
           *p2 = *p1;
           p2++;
         }
         p1++;
    }
    *p2='\0';

}
/* find_pid_by_name()
 *
 *  This finds the pid of the specified process.
 *  Currently, it's implemented by rummaging through
 *  the proc filesystem.
 *
 *  Returns a list of all matching PIDs
 */
static pid_t* find_pid_by_name( char* pidName)
{
        DIR *dir;
        struct dirent *next;
        pid_t* pidList=NULL;
        int i=0;

        dir = opendir("/proc");
        if (!dir) {
                printf("cfm:Cannot open /proc");
                return NULL;
        }

        while ((next = readdir(dir)) != NULL) {
                /*FILE *status */
                FILE *cmdline;
                char filename[READ_BUF_SIZE];
                char buffer[READ_BUF_SIZE];
                /* char name[READ_BUF_SIZE]; */

                /* Must skip ".." since that is outside /proc */
                if (strcmp(next->d_name, "..") == 0)
                        continue;

                /* If it isn't a number, we don't want it */
                if (!isdigit(*next->d_name))
                        continue;

                /* sprintf(filename, "/proc/%s/status", next->d_name); */
                /* read /porc/<pid>/cmdline instead to get full cmd line */
                sprintf(filename, "/proc/%s/cmdline", next->d_name);
                if (! (cmdline = fopen(filename, "r")) ) {
                        continue;
                }
                if (fgets(buffer, READ_BUF_SIZE-1, cmdline) == NULL) {
                        fclose(cmdline);
                        continue;
                }
                fclose(cmdline);

                /* Buffer should contain a string like "Name:   binary_name" */
                /*sscanf(buffer, "%*s %s", name);*/
                /* buffer contains full commandline params separted by '\0' */
                remove_delimitor(buffer);
                if (strstr(buffer, pidName) != NULL) {
                        pidList=realloc( pidList, sizeof(pid_t) * (i+2));
                        if (!pidList) {
                                printf("cfm: Out of memeory!\n");
				closedir(dir);
                                return NULL;
                        }
                        pidList[i++]=strtol(next->d_name, NULL, 0);
                }
        }
        closedir(dir);

        if (pidList)
                pidList[i]=0;
        else if ( strcmp(pidName, "init")==0) {
                /* If we found nothing and they were trying to kill "init",
                 * guess PID 1 and call it good...  Perhaps we should simply
                 * exit if /proc isn't mounted, but this will do for now. */
                pidList=realloc( pidList, sizeof(pid_t));
                if (!pidList) {
                        printf("cfm: Out of memeory!\n");
                        return NULL;
                }
                pidList[0]=1;
        } else {
                pidList=realloc( pidList, sizeof(pid_t));
                if (!pidList) {
                        printf("cfm: Out of memeory!\n");
                        return NULL;
                }
                pidList[0]=-1;
        }
        return pidList;
}

#endif


/***************************************************************************
// Function Name: bcmSystem().
// Description  : launch shell command in the child process.
// Parameters   : command - shell command to launch.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmSystemEx (char *command, int printFlag) {
   int pid = 0, status = 0;

   if ( command == 0 )
      return 1;

   pid = fork();
   if ( pid == -1 )
      return -1;

   if ( pid == 0 ) {
      char *argv[4];
      argv[0] = "sh";
      argv[1] = "-c";
      argv[2] = command;
      argv[3] = 0;
#ifdef BRCM_DEBUG
      if (printFlag)
         printf("app: %s\r\n", command);
#endif
      if (printFlag)
        syslog(LOG_DEBUG, command);
      execve("/bin/sh", argv, environ);
      exit(127);
   }

   /* wait for child process return */
   do {
      if ( waitpid(pid, &status, 0) == -1 ) {
         if ( errno != EINTR )
            return -1;
      } else
         return status;
   } while ( 1 );

   return status;
}

/***************************************************************************
// Function Name: bcmCreateDhcpCfg().
// Description  : create DHCP server configuration file.
// Parameters   : ipAddr - IP address of target.
//                mask - subnet mask of target.
//                addrStart - start IP address of DHCP server pool.
//                addrEnd - end IP address of DHCP server pool.
//                dns1 - primary dns.
//                dns2 - secondary dns.
//                leasedTime - leased time.
//                protoInfo - current network protocol.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmCreateDhcpCfg(char *ipAddr, char *mask,
                     char *addrStart, char *addrEnd,
                     char *dns1, char *dns2, int leasedTime,
                     int protocol, int enblFirewall) {
   FILE* fs = fopen("/etc/udhcpd.conf", "w");

   if ( fs != NULL ) {
      // cwu
      char cmd[128]="";
      sprintf(cmd, "echo %s > /var/run/ip", addrStart);
      system(cmd);

      // start IP address
      sprintf(cmd, "start %s\n", addrStart);
      fputs(cmd, fs);

      // end IP address
      sprintf(cmd, "end %s\n", addrEnd);
      fputs(cmd, fs);

      // interface
      switch ( protocol ) {
      case PROTO_PPPOA:
      case PROTO_PPPOE:
      case PROTO_MER:
      case PROTO_IPOA:
      case PROTO_NONE:
         fputs("interface br0\n", fs);
         break;
      }

	  // lease
      sprintf(cmd, "option lease %d\n", leasedTime);
      fputs(cmd, fs);
      sprintf(cmd, "option min_lease 30\n");
      fputs(cmd, fs);

      // subnet mask
      sprintf(cmd, "option subnet %s\n", mask);
      fputs(cmd, fs);

      // router
      sprintf(cmd, "option router %s\n", ipAddr);
      fputs(cmd, fs);

      // use DNS relay when firewall is enable
      if ( enblFirewall == 1 ) {
         // always use DSL router IP address as DNS
         // for DHCP server since we want local PCs
         // use DHCP server relay. The real DNS is
         // stored in /etc/resolv.conf
         sprintf(cmd, "option dns %s\n", ipAddr);
         fputs(cmd, fs);
      } else { // use real DNS when firewall is disable
         // primary DNS
         if ( strcmp(dns1, "0.0.0.0") != 0 )
            sprintf(cmd, "option dns %s\n", dns1);
         else
            sprintf(cmd, "option dns %s\n", ipAddr);
         fputs(cmd, fs);
         // secondary DNS
         if ( strcmp(dns2, "0.0.0.0") != 0 )
            sprintf(cmd, "option dns %s\n", dns2);
         else
            sprintf(cmd, "option dns %s\n", ipAddr);
         fputs(cmd, fs);
      }

//      fputs("option lease 60", fs);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmCreateIpExtDhcpCfg().
// Description  : create DHCP server configuration file for PPP IP Extenstion.
// Parameters   : lanAddr - LAN IP address of target.
//                mask - subnet mask of target.
//                wanAddr - WAN IP address.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmCreateIpExtDhcpCfg(char *lanAddr, char *mask, char *wanAddr) {
   char cmd[SYS_CMD_LEN], dns[SYS_CMD_LEN];
   FILE* fs = fopen("/etc/udhcpd.conf", "w");

   if ( fs != NULL ) {
       // cwu
      sprintf(cmd, "echo %s > /var/run/ip", wanAddr);
      system(cmd);


      // start IP address
      sprintf(cmd, "start %s\n", wanAddr);
      fputs(cmd, fs);

      // end IP address
      sprintf(cmd, "end %s\n", wanAddr);
      fputs(cmd, fs);

      // interface
      fputs("interface br0\n", fs);

      // lease
      fputs("option lease 30\n", fs);
      fputs("option min_lease 30\n", fs);

      // subnet mask
      sprintf(cmd, "option subnet %s\n", mask);
      fputs(cmd, fs);

      // router
      sprintf(cmd, "option router %s\n", wanAddr);
      fputs(cmd, fs);

      // don't use DNS relay since there is no ip table for
      // PPP IP extension
      bcmGetDns(dns);
      sprintf(cmd, "option dns %s\n", dns);
      fputs(cmd, fs);

      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmSetIpExtInfo().
// Description  : store wan, gateway, and dns for PPP IP extension.
// Parameters   : wan - WAN IP address.
//                gateway - default gateway.
//                dns - dns.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmSetIpExtInfo(char *wan, char *gateway, char *dns) {
   char str[256];
   FILE* fs = fopen("/var/ipextinfo", "w");

   if ( fs != NULL ) {
      sprintf(str, "%s %s %s\n", wan, gateway, dns);
      fputs(str, fs);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmGetIpExtInfo().
// Description  : get wan, gateway, or dns for PPP IP extension.
// Parameters   : buf - .
//                type - .
// Returns      : none.
****************************************************************************/
void bcmGetIpExtInfo(char *buf, int type) {
   FILE* fs;
   char wan[64], gateway[64], dns[64], str[256];

   if ( buf == NULL ) return;

   buf[0] = '\0';
   if( bcmGetAdslStatus() == MNTR_STS_OK ) {
      fs = fopen("/var/ipextinfo", "r");
      if ( fs != NULL ) {
         fgets(str, 256, fs);
         fclose(fs);
         sscanf(str, "%s %s %s\n", wan, gateway, dns);
         switch ( type ) {
         case 0:
            if( dns[0] >= '0' && dns[0] <= '9' )
               strcpy(buf, wan);
            break;
         case 1:
            if( dns[0] >= '0' && dns[0] <= '9' )
               strcpy(buf, gateway);
            break;
         case 2:
            if( dns[0] >= '0' && dns[0] <= '9' )
               strcpy(buf, dns);
            break;
         }
      }
   }
}

/***************************************************************************
// Function Name: bcmCreateLocalDhcpCfg().
// Description  : create DHCP server configuration file with default local.
// Parameters   : ipAddr -- default local IP address.
//                mask -- default local subnet mask.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmCreateLocalDhcpCfg(char *ipAddr, char *mask) {
   FILE* fs = fopen("/etc/udhcpd.conf", "w");
   char cmd[SYS_CMD_LEN], nextAddr[SYS_CMD_LEN];
   struct in_addr addr;

   if ( fs != NULL ) {
      addr.s_addr = inet_addr(ipAddr) + 1;
      strcpy(nextAddr, inet_ntoa(addr));

      sprintf(cmd, "echo %s > /var/run/ip", nextAddr);
      system(cmd);

      // start IP address
      fprintf(fs, "start %s\n", nextAddr);

      // end IP address
      fprintf(fs, "end %s\n", nextAddr);

      // interface
      fputs("interface br0\n", fs);

      // lease
      fputs("option lease 10\n", fs);
      fputs("option min_lease 10\n", fs);

      // subnet mask
      fprintf(fs, "option subnet %s\n", mask);

      // router
      fprintf(fs, "option router %s\n", ipAddr);

      // dns
      fprintf(fs, "option dns %s\n", ipAddr);

      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmCreateResolvCfg().
// Description  : create resolv configuration file.
// Parameters   : dns1 - primary dns.
//                dns2 - secondary dns.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmCreateResolvCfg(char *dns1, char *dns2) {
   char cmd[SYS_CMD_LEN];
   FILE* fs = NULL;

   bcmSystemMute("mkdir -p /var/fyi/sys");
   bcmSystemMute("echo > /var/fyi/sys/dns");
   fs = fopen("/var/fyi/sys/dns", "w");

   if ( fs != NULL ) {
      sprintf(cmd, "nameserver %s\n", dns1);
      fputs(cmd, fs);
      sprintf(cmd, "nameserver %s\n", dns2);
      fputs(cmd, fs);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

// global ADSL info variable is declared here (in syscall.c)
// and is used in syscall.c, sysdiag.c, cgimain.c, and cgists.c
ADSL_CONNECTION_INFO glbAdslInfo;

/***************************************************************************
// Function Name: bcmGetAdslStatus().
// Description  : get ADSL status.
// Parameters   : none
// Returns      : 0 - ADSL link Up (OK)
//                1 - ADSL link Down
//                2 - Other error
****************************************************************************/
int bcmGetAdslStatus() {
   int ret = 0;

   if (BcmAdslCtl_GetConnectionInfo(&glbAdslInfo) != BCMADSL_STATUS_ERROR) {
      ret = glbAdslInfo.LinkState;
//      if ( glbAdslInfo.LinkState == BCM_ADSL_LINK_UP )
//         ret = ADSL_LINK_UP;
//      else
//         ret = ADSL_LINK_DOWN;
   } else
      ret = BCMADSL_STATUS_ERROR;

   return ret;
}

/***************************************************************************
// Function Name: bcmGetPppStatus().
// Description  : get PPP status.
// Parameters   : str - buffer to retrieve message
//                len - length of buffer
// Returns      : 0 - OK
//                -1 - ERROR
****************************************************************************/
int bcmGetPppStatus(char *str, int len, char *name) {
   FILE* fs;

   char filePath[100];
   sprintf(filePath,"/proc/var/fyi/wan/%s/daemonstatus",name);
   fs = fopen(filePath, "r");
   if ( fs != NULL ) {
      fgets(str, len, fs);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmGetDhcpcStatus().
// Description  : get DHCPC status.
// Parameters   : str - buffer to retrieve message
//                len - length of buffer
// Returns      : 0 - OK
//                -1 - ERROR
****************************************************************************/
int bcmGetDhcpcStatus(char *str, int len) {
   FILE* fs = fopen("/var/run/dhcpc", "r");

   if ( fs != NULL ) {
      fgets(str, len, fs);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmGetSystemStatus().
// Description  : get system status.
// Parameters   : str - buffer to retrieve message
//                len - length of buffer
// Returns      : 0 - OK
//                -1 - ERROR
****************************************************************************/
int bcmGetSystemStatus(char *str, int len) {
   FILE* fs = fopen("/etc/sysmsg", "r");

   if ( fs != NULL ) {
      fgets(str, len, fs);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmSetSystemStatus().
// Description  : set system status.
// Parameters   : int - system status
// Returns      : 0 - OK
//                -1 - ERROR
****************************************************************************/
int bcmSetSystemStatus(int status) {
   char cmd[SYS_CMD_LEN];
   FILE* fs = fopen("/etc/sysmsg", "w");

   if ( fs != NULL ) {
      sprintf(cmd, "%d\n", status);
      fputs(cmd, fs);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmDisplayLed().
// Description  : display LED corresponding to WAN link status.
// Parameters   : status - WAN link status.
// Returns      : none.
****************************************************************************/
void bcmDisplayLed(int status) {
   switch (status) {
   case MNTR_STS_ADSL_DOWN:
      /* this means ADSL is DOWN */
//      sysLedCtrl(kLedPPP, kLedStateOn);    // PPP is GPIO_10 and is used as Alarm now
      sysLedCtrl(kLedPPP, kLedStateOff);     // USR9106: PPP is GPIO_7 and is used to indicate PPP failure.
      sysLedCtrl(kLedWanData, kLedStateOff); // USR9106: Wan led off until ADSL is up.
      break;
   case MNTR_STS_ADSL_TRAINING:
      /* this means ADSL is TRAINING */
//      sysLedCtrl(kLedPPP, kLedStateOn);    // adsl is not up yet, alarm on
      sysLedCtrl(kLedPPP, kLedStateOff);     // USR9106: Wan led off until ADSL is up.
      sysLedCtrl(kLedWanData, kLedStateOff);
      break;
   case MNTR_STS_PPP_AUTH_ERR:
   case MNTR_STS_PPP_DOWN:
      /* this means ADSL is UP, but not PPP */
//      sysLedCtrl(kLedPPP, kLedStateOff);   // adsl is up, alarm off
      sysLedCtrl(kLedPPP, kLedStateOn);      // USR9106: Error, turn Red led On.
      sysLedCtrl(kLedWanData, kLedStateOff); // Wan green led Off
      break;
   case MNTR_STS_OK:
      /* this means ADSL and PPP are up */
//      sysLedCtrl(kLedPPP, kLedStateOff);     // adsl is up, alarm off
      break;
   }
}

/***************************************************************************
// Function Name: bcmGetDns().
// Description  : get DSN info.
// Parameters   : dns - buffer to retrieve primary dns.
// Returns      : none.
****************************************************************************/
void bcmGetDns(char *dns) {
   char str[SYS_CMD_LEN];
   FILE* fs = NULL;
   UINT16 retLen = 0;
   IFC_DNS_INFO dnsInfo;
   PSI_HANDLE sysPsi = BcmPsi_appOpen(SYSTEM_APPID);

   dns[0] = '\0';
   PSI_STATUS status = BcmPsi_objRetrieve(sysPsi, SYS_DNS_ID, &dnsInfo,
                                          sizeof(dnsInfo), &retLen);
   if ( status == PSI_STS_OK ) {
      if ( dnsInfo.preferredDns.s_addr != INADDR_NONE )
         strcpy(dns, inet_ntoa(dnsInfo.preferredDns));
   } else {
      fs = fopen("/var/fyi/sys/dns", "r");
      if ( fs != NULL ) {
         if ( fgets(str, SYS_CMD_LEN, fs) > 0 )
            sscanf(str, "nameserver %s\n", dns);
         fclose(fs);
      } else
         // if cannot find primary dns info then
         // assign default value which is router IP address
         bcmGetIfDestAddr("br0", dns);
   }
}

/***************************************************************************
// Function Name: bcmGetDns2().
// Description  : get DSN info.
// Parameters   : dns - buffer to retrieve primary dns.
// Returns      : none.
****************************************************************************/
void bcmGetDns2(char *dns) {
   char str[SYS_CMD_LEN];
   FILE* fs = NULL;
   UINT16 retLen = 0;
   IFC_DNS_INFO dnsInfo;
   PSI_HANDLE sysPsi = BcmPsi_appOpen(SYSTEM_APPID);

   dns[0] = '\0';
   PSI_STATUS status = BcmPsi_objRetrieve(sysPsi, SYS_DNS_ID, &dnsInfo, sizeof(dnsInfo), &retLen);
   if ( status == PSI_STS_OK ) {
      if ( dnsInfo.alternateDns.s_addr != INADDR_NONE )
         strcpy(dns, inet_ntoa(dnsInfo.alternateDns));
   } else {
      fs = fopen("/var/fyi/sys/dns", "r");
      if ( fs != NULL ) {
         if ( fgets(str, SYS_CMD_LEN, fs) > 0 ) {
            if ( fgets(str, SYS_CMD_LEN, fs) > 0 )
               sscanf(str, "nameserver %s\n", dns);      // get second one if exist
         }
         fclose(fs);
      } else
         // if cannot find secondary dns info then
         // assign default value which is router IP address
         bcmGetIfDestAddr("br0", dns);
   }
}

/***************************************************************************
// Function Name: bcmRestartDnsProbe().
// Description  : start DNS probe.
// Parameters   : none.
// Returns      : none.
****************************************************************************/
void bcmRestartDnsProbe(void) {
   char cmd[CLI_MAX_BUF_SZ];

   // kill the old dnsprobe if it is existed
   int pid = bcmGetPid("/bin/dnsprobe");
   if ( pid > 0 ) {
      sprintf(cmd, "kill -9 %d", pid);
      bcmSystem(cmd);
   }

   // start the new dnsprobe
   bcmSystem("/bin/dnsprobe &");
}

/***************************************************************************
// Function Name: bcmConfigDns().
// Description  : add or remove DNS info to PSI.
// Parameters   : primary and secondary DNS.
// Returns      : none.
****************************************************************************/
void bcmConfigDns(char *dns1, char *dns2) {
   char cmd[CLI_MAX_BUF_SZ], addr[CLI_MAX_BUF_SZ], buf[CLI_MAX_BUF_SZ];
   UINT16 retLen = 0;
   IFC_DNS_INFO dnsInfo;
   PSI_STATUS status = PSI_STS_OK;
   PSI_HANDLE sysPsi = BcmPsi_appOpen(SYSTEM_APPID);

   // get local ip address
   bcmGetIfDestAddr("br0", addr);

   // if user changes from static to auto assgined dns
   if ( ( strcmp(dns1, "0.0.0.0") == 0 ||
          strcmp(dns1, "") == 0 ) &&
        ( strcmp(dns2, "0.0.0.0") == 0 ||
          strcmp(dns2, "") == 0 ) ) {
      status = BcmPsi_objRetrieve(sysPsi, SYS_DNS_ID, &dnsInfo, sizeof(dnsInfo), &retLen);
      if ( status == PSI_STS_OK &&
           bcmIsModuleInserted("iptable_nat") == TRUE ) {
         strcpy(buf, inet_ntoa(dnsInfo.preferredDns));
         // del the old rule
         sprintf(cmd, "iptables -t nat -D PREROUTING -i br0 -d %s -p udp --dport 53 -j DNAT --to %s", addr, buf);
         bcmSystemMute(cmd);
      }
      BcmPsi_objRemove(sysPsi, SYS_DNS_ID);
   } else {
      // create the new resolv.conf with new dsn info
      bcmCreateResolvCfg(dns1, dns2);
      // get old dns1 info
      bcmGetDns(buf);
      if ( buf[0] != '\0' ) {
         if (strcmp(buf, dns1) != 0) {
            if ( bcmIsModuleInserted("iptable_nat") == TRUE ) {
               // del the old rule
               sprintf(cmd, "iptables -t nat -D PREROUTING -i br0 -d %s -p udp --dport 53 -j DNAT --to %s", addr, buf);
               bcmSystemMute(cmd);
               // add new DNS Forwarding rule
               sprintf(cmd, "iptables -t nat -A PREROUTING -i br0 -d %s -p udp --dport 53 -j DNAT --to %s", addr, dns1);
               bcmSystemMute(cmd);
               bcmRestartDnsProbe();
            }
         }
      } else {
         printf("No Existing DNS information from DSL router\n");
         sprintf(cmd, "iptables -t nat -A PREROUTING -i br0 -d %s -p udp --dport 53 -j DNAT --to %s", addr, dns1);
         bcmSystemMute(cmd);
      }
      // get old dns2 info
      bcmGetDns2(buf);
      // if old dns2 differs with new one, restart dnsprobe
      if (strcmp(buf, dns2) != 0) {
         if ( bcmIsModuleInserted("iptable_nat") == TRUE )
            bcmRestartDnsProbe();
      }
      dnsInfo.preferredDns.s_addr = inet_addr(dns1);
      dnsInfo.alternateDns.s_addr = inet_addr(dns2);
      BcmPsi_objStore(sysPsi, SYS_DNS_ID, &dnsInfo, sizeof(dnsInfo));
   }

   BcmPsi_flush();
}

/***************************************************************************
// Function Name: bcmGetDefaultGateway().
// Description  : get default gateway info.
// Parameters   : gtwy - buffer to retrieve default gateway.
// Returns      : none.
****************************************************************************/
void bcmGetDefaultGateway(char *gtwy, char *wanIf) {
   char str[SYS_CMD_LEN];
   FILE* fs = NULL;
   UINT16 retLen = 0;
   IFC_DEF_GW_INFO defgw;
   char addr[512], ip[512];

   PSI_HANDLE sysPsi = BcmPsi_appOpen(SYSTEM_APPID);

   gtwy[0] = wanIf[0] = '\0';
   PSI_STATUS status = BcmPsi_objRetrieve(sysPsi, SYS_DFT_GTWY_ID,
                                          &defgw, sizeof(defgw), &retLen);
   if ( status == PSI_STS_OK ){
      if (strcmp(inet_ntoa(defgw.defaultGateway), "0.0.0.0") != 0)
         strcpy(gtwy, inet_ntoa(defgw.defaultGateway));
      if (defgw.ifName != NULL)
         strcpy(wanIf, defgw.ifName);      
   }
   else {
      fs = fopen("/var/fyi/sys/gateway", "r");
      if ( fs != NULL ) {
         if ( fgets(str, SYS_CMD_LEN, fs) > 0 ) {
            sscanf(str, "%s\n", addr);
            if ( BcmCli_isIpAddress(addr) == CLI_TRUE ) 
               strcpy(gtwy, addr);
            else { // use ifName
               if (bcmGetIfDestAddr(addr, ip) == BCM_DIAG_PASS)
                  strcpy(gtwy, ip); 
            }
         }
         fclose(fs);
      }
   }
}

/***************************************************************************
// Function Name: bcmIsDefaultGatewayExisted().
// Description  : Check if the defaultGateway is existed in the route table.
// Parameters   : gw and wanIf
// Returns      : 1 - Exist. 0 - not Exist
****************************************************************************/
int bcmIsDefaultGatewayExisted(char *gw, char *wanIf) {
   char col[11][32];
   char line[512];
   struct in_addr addr;
   int count = 0;
   int flag = 0;

   FILE* fsRoute = fopen("/proc/net/route", "r");
   if ( fsRoute != NULL ) {
      while ( fgets(line, sizeof(line), fsRoute) ) {
         // read pass header line
        if ( count++ < 1 ) continue;
        sscanf(line, "%s %s %s %s %s %s %s %s %s %s %s",
               col[0], col[1], col[2], col[3], col[4], col[5],
               col[6], col[7], col[8], col[9], col[10]);
        flag = strtol(col[3], (char**)NULL, 16);
        if ((flag & (RTF_UP | RTF_GATEWAY)) == (RTF_UP | RTF_GATEWAY)) {
           if ( wanIf[0] == '\0' || strcmp(wanIf, col[0]) == 0) {
             addr.s_addr = strtoul(col[2], (char**)NULL, 16);
             if (strcmp(gw, inet_ntoa(addr)) == 0) {
                fclose(fsRoute);
                return TRUE;
             }
           }
        }
      }
      fclose(fsRoute);
   }

   return FALSE;
}

/***************************************************************************
// Function Name: bcmSetAutoDefaultGateway().
// Description  : remove static default gateway in PSI,
//                get the default gateway from var/fyi/sys/gateway, and add
//                it to the route table.
// Parameters   : errMsg -- error message when add default gateway if any.
// Returns      : 1 - Exist. 0 - not Exist
****************************************************************************/
void bcmSetAutoDefaultGateway(char *errMsg) {
   char sysDefaultGateway[IFC_TINY_LEN];
   char cmd[IFC_LARGE_LEN];
   FILE *fs = NULL;
   UINT16 retLen = 0;
   IFC_DEF_GW_INFO defgw;
   PSI_STATUS status;
   PSI_HANDLE sysPsi = BcmPsi_appOpen(SYSTEM_APPID);

   // remove static default gateway
   status = BcmPsi_objRetrieve(sysPsi, SYS_DFT_GTWY_ID, &defgw, sizeof(defgw), &retLen);
   if ( status == PSI_STS_OK ) {
      // remove static default gateway in PSI
      BcmPsi_objRemove(sysPsi, SYS_DFT_GTWY_ID);
      BcmPsi_flush();
      // create command to delete default gateway in route table
      sprintf(cmd, "route del default");
      if (strcmp(inet_ntoa(defgw.defaultGateway), "0.0.0.0") != 0) {
         strcat(cmd, " gw ");
         strcat(cmd, inet_ntoa(defgw.defaultGateway));
      }
      if (strcmp(defgw.ifName, "") != 0) {
         strcat(cmd, " dev ");
         strcat(cmd, defgw.ifName);
      }
      strcat(cmd, " 2> /var/gwerr");
      bcmSystem(cmd);
      fs = fopen("/var/gwerr", "r");
      // read gwerr, if there is err then
      // need to set error message
      if ( fs != NULL ) {
         if ( fgets(cmd, IFC_LARGE_LEN, fs) > 0 ) {
            // remove the last newline character
            cmd[strlen(cmd) - 1] = '\0';
            strcpy(errMsg, cmd);
            bcmSystemMute("cat /var/gwerr");
         } else
            errMsg[0] = '\0';
         fclose(fs);
         bcmSystemMute("rm /var/gwerr");
      }
   }

   //get the default gateway from var/fyi/sys/gateway
   bcmGetDefaultGateway(sysDefaultGateway, cmd);
   if ( sysDefaultGateway[0] != '\0' &&
        bcmIsDefaultGatewayExisted(sysDefaultGateway, "") == FALSE ) {
      sprintf(cmd, "route add default gw %s 2> /var/gwerr", sysDefaultGateway);
      bcmSystem(cmd);
      fs = fopen("/var/gwerr", "r");
      // read gwerr, if there is err then
      // need to set the error message
      if ( fs != NULL ) {
         if ( fgets(cmd, IFC_LARGE_LEN, fs) > 0 ) {
            // remove the last newline character
            cmd[strlen(cmd) - 1] = '\0';
            strcpy(errMsg, cmd);
            bcmSystemMute("cat /var/gwerr");
         } else
            errMsg[0] = '\0';
         fclose(fs);
         bcmSystemMute("rm /var/gwerr");
      }
   }
}

/***************************************************************************
// Function Name: bcmSetStaticDefaultGateway().
// Description  : remove the old, and add the new default gateway.
// Parameters   : gw, wanIf, and error message
// Returns      : none
****************************************************************************/
void bcmSetStaticDefaultGateway(char *gw, char *wanIf, char *errMsg) {
   int runCmd = FALSE;
   char cmd[IFC_LARGE_LEN];
   FILE* fs = NULL;
   UINT16 retLen = 0;
   IFC_DEF_GW_INFO defgw;
   PSI_STATUS status;
   PSI_HANDLE sysPsi = BcmPsi_appOpen(SYSTEM_APPID);

   // intialize gw if it is empty
   if ( gw[0] == '\0' )
      strcpy(gw, "0.0.0.0");

   status = BcmPsi_objRetrieve(sysPsi, SYS_DFT_GTWY_ID, &defgw, sizeof(defgw), &retLen);
   if ( status == PSI_STS_OK ) {
      // check if not the same
      if ( strcmp(defgw.ifName, wanIf) != 0 ||
           strcmp(inet_ntoa(defgw.defaultGateway), gw) != 0) {
         // del the previous default route saved in route table
         sprintf(cmd, "route del default");
         if ( strcmp(inet_ntoa(defgw.defaultGateway), "0.0.0.0") != 0 ) {
            strcat(cmd, " gw ");
            strcat(cmd, inet_ntoa(defgw.defaultGateway));
         }
         if ( strcmp(defgw.ifName, "") != 0 ) {
            strcat(cmd, " dev ");
            strcat(cmd, defgw.ifName);
         }
         strcat(cmd, " 2> /var/gwerr");
         bcmSystemMute(cmd);
         fs = fopen("/var/gwerr", "r");
         // read gwerr, if there is err then
         // need to set error message
         if ( fgets(cmd, IFC_LARGE_LEN, fs) > 0 ) {
            // remove the last newline character
            cmd[strlen(cmd) - 1] = '\0';
            strcpy(errMsg, cmd);
            bcmSystemMute("cat /var/gwerr");
         } else
            errMsg[0] = '\0';
         fclose(fs);
         bcmSystemMute("rm /var/gwerr");
      }
      else
         return; // same and do nothing
   }

   // if this gateway and wanIf is already in the route table,
   // do not issue route add default command.
   if ( bcmIsDefaultGatewayExisted(gw, wanIf) == FALSE ) {
      strcpy(cmd, "route add default");
      if ( strcmp(gw, "0.0.0.0") != 0 &&
           BcmCli_isIpAddress(gw) == CLI_TRUE ) {
         strcat(cmd, " gw ");
         strcat(cmd, gw);
         runCmd = TRUE;
      }
      if (wanIf[0] != '\0') {
         strcat(cmd, " dev ");
         strcat(cmd, wanIf);
         runCmd = TRUE;
      }
      if ( runCmd == TRUE ) {
         strcat(cmd, " 2> /var/gwerr");
         bcmSystemMute(cmd);
         fs = fopen("/var/gwerr", "r");
         // read gwerr, if there is no err then
         // need to configure default gateway in PSI
         if ( fgets(cmd, IFC_LARGE_LEN, fs) <= 0 ) {
            // save the new default gateway info to psi
            errMsg[0] = '\0';
         } else {
            // remove the last newline character
            cmd[strlen(cmd) - 1] = '\0';
            strcpy(errMsg, cmd);
            bcmSystemMute("cat /var/gwerr");
         }
         // save to PSI even when error occurs
         defgw.enblGwAutoAssign = 0;
         strcpy(defgw.ifName, wanIf);
         defgw.defaultGateway.s_addr = inet_addr(gw);
         BcmPsi_objStore(sysPsi, SYS_DFT_GTWY_ID, &defgw, sizeof(defgw));
         BcmPsi_flush();
         // close gwerr file
         fclose(fs);
         bcmSystemMute("rm /var/gwerr");
      }
   }
}

//**************************************************************************
// Function Name: getPppoeServiceName
// Description  : get pppoe service name of the specific wan interfaces.
// Parameters   : service -- pppoe service name. (output)
//                ifName -- interface name. (input)
// Returns      : none.
//**************************************************************************
void getPppoeServiceName(char *service, char *ifName) {
   char fileName[IFC_LARGE_LEN];
   char str[SYS_CMD_LEN];
   FILE* fs = NULL;

   if (ifName[0] != 0) {
      sprintf(fileName, "/proc/var/fyi/wan/%s/servicename", ifName);
      fs = fopen(fileName, "r");
      if (fs != NULL) {
         if (fgets(str, SYS_CMD_LEN, fs) > 0)
            sscanf(str, "%s\n", service);
	 fclose(fs);
      }
   }
}

/***************************************************************************
// Function Name: bcmGetPid().
// Description  : get process PID by using process name.
// Parameters   : command - command that launch the process.
// Returns      : process ID number.
****************************************************************************/
int bcmGetPid(char * command)
{
    char cmdline[128], *p1, *p2;
    pid_t *pid = NULL;
    int ret = 0;

    p1 = command;
    p2 = cmdline;
    while ( *p1 != '\0') {
        if (*p1 != ' ') {
           *p2 = *p1;
           p2++;
         }
         p1++;
    }
    *p2='\0';

    pid = find_pid_by_name(cmdline);
    if ( pid != NULL ) {
       ret = (int)(*pid);
       free(pid);
    }

    return ret;
}

/***************************************************************************
// Function Name: bcmCheckInterfaceUp().
// Description  : check status of interface.
// Parameters   : devname - name of device.
// Returns      : 1 - UP.
//                0 - DOWN.
****************************************************************************/
int bcmCheckInterfaceUp(char *devname) {
   int  skfd;
   int  ret;
   struct ifreq intf;

   if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      return 0;
   }

   strcpy(intf.ifr_name, devname);
   if (ioctl(skfd, SIOCGIFFLAGS, &intf) == -1) {
      ret = 0;
   } else {
      if ( (intf.ifr_flags & IFF_UP) != 0)
         ret = 1;
      else
         ret = 0;
   }

   close(skfd);
   return ret;
}

/***************************************************************************
// Function Name: setWanLinkStatus().
// Description  : get PPP status.
// Parameters   : up - wan link status
// Returns      : none
//                -1 - ERROR
****************************************************************************/
void setWanLinkStatus(int up) 
{
   FILE * fd = NULL;

   if ( up == 1 ) {
      if ( (fd = fopen("/var/run/wanup", "a+")) != NULL )
         fclose(fd);
   } else
      unlink("/var/run/wanup");
}

/***************************************************************************
// Function Name: disconnectPPP().
// Description  : if PPPD is alive, disconnet it.
// Returns      : none
****************************************************************************/
void disconnectPPP(void)
{
   int pid = 0;

   // If PPP is connected disconnect it first
   if ( (pid = bcmGetPid("pppd")) > 0 ) {
      setWanLinkStatus(0);
      // give PPP some time to disconnect. PPP client checks WAN links status
      // every 3 seconds 
      sleep(6);
   }
}


/***************************************************************************
// Function Name: bcmSocketIfPid().
// Description  : return the socket if pid
// Parameters   : none.
// Returns      : pid
****************************************************************************/
int bcmSocketIfPid(FILE *fs)
{
   char ifName[32];
   int pid = 0;
   char buf[SYS_CMD_LEN];
   char *ptr, *ptr2;
   
   // Check this first
   strcpy(ifName, getIfName());
   if (ifName[0] == '\0')
      return pid;

   if (strstr(ifName, "nas") != NULL) {    // its a MER 
      while (fgets(buf, SYS_CMD_LEN, fs) > 0) 
         if ((ptr = strstr(buf, "dhcpc")) != NULL && (ptr2 = strstr(buf, ifName)) != NULL) { // found command line 
            *(ptr2+5) = '\0';     // 'nas??\0', null the cmd
            pid = bcmGetPid(ptr);
         }
   }
   else if (strstr(ifName, "ppp") != NULL) { // its pppoe or pppoa
      char endCmd[5];
      endCmd[0] = ' ';           // eg. ' 35'
      strcpy(&endCmd[1], &ifName[3]);
      while (fgets(buf, SYS_CMD_LEN, fs) > 0) 
         if ((ptr = strstr(buf, "pppd")) != NULL && (ptr2 = strstr(buf, endCmd)) != NULL) {  // found command lin
            *(ptr2+3) = '\0';     // ' 35\0' null the cmd
            pid =bcmGetPid(ptr);
         }
   }
   
   return pid;
}



/***************************************************************************
// Function Name: bcmKillAllApps().
// Description  : kill all available applications.
// Parameters   : none.
// Returns      : none.
****************************************************************************/
void bcmKillAllApps(void) {
   // NOTE: Add the app name before NULL. Order is important
   char *apps[]=
   {
      "telnetd",   
      "upnp",
      "reaim",
      "klogd",
      "syslogd",
      "ripd",
      "zebra",
      "igmp",
      "dnsprobe",
      "pppd",
      "dhcpc",
      NULL,
   };
   char cmd[SYS_CMD_LEN], app[SYS_CMD_LEN], buf[SYS_CMD_LEN];
   char pidStr[SYS_CMD_LEN];
   int pid = 0;
   int i = 0;
   FILE *fs;
   int socketPid = 0;
   int curPid = getpid();
   int httpdPid =0;
   int inWeb = FALSE;
   int killNo = 0;

   fs = fopen("/var/run/httpd_pid", "r");
   if (fs != NULL)
   {
      fgets(cmd, SYS_CMD_LEN, fs);
      httpdPid = atoi(cmd);
      fclose(fs);
   }
   else
   {
      printf("Error: httpd pid not found ?\n");
      return;
   }

   bcmSystemMute("ps > /var/pslist");
   fs = fopen("/var/pslist", "r");
   if (fs == NULL)
      return;
   bcmSystem("cat /var/pslist");

   socketPid = bcmSocketIfPid(fs);
   if (socketPid != 0) 
   {
      // add the default route with the ifc the socket is sitting on
      sprintf(cmd, "route add default dev %s", getIfName());
      bcmSystem(cmd);
      rewind(fs);
   }    

   if (curPid == httpdPid)
      inWeb = TRUE;

   //printf("curPid=%d, socketPid=%d, htpdPid=%d, inWeb=%d\n", curPid, socketPid, httpdPid, inWeb);
   // get app and read thru pslist according to the order of apps
   i = 0;
   do 
   {
      strcpy(app, apps[i]);
      while (fgets(buf, SYS_CMD_LEN, fs) > 0)  
      {
         if (strstr(buf, app) != NULL) // found command line with match app name
         {  
            // find pid string
            sscanf(buf, "%s\n", pidStr);
            // if not in httpd, must be in telnetd and skip the line.
            if (!inWeb && strstr(buf, "telnetd"))
               break;
            pid = atoi(pidStr);
            if (pid != curPid && pid != socketPid)
            {
               printf("kill %s process...\n", app);
               killNo = 9;
               if (strcmp(app, "zebra") == 0 || strcmp(app, "ripd") == 0)
                  killNo = 15;
               sprintf(cmd, "kill -%d %d", killNo, pid);
               bcmSystem(cmd);
            }
         }
      } 
      rewind(fs);    // start pslist over again
   } while (apps[++i] != NULL);

   fclose(fs);
   bcmSystemMute("rm /var/pslist");
}

// function to support encryption password for login
static int i64c(int i) {
    if (i <= 0)
        return ('.');
    if (i == 1)
        return ('/');
    if (i >= 2 && i < 12)
        return ('0' - 2 + i);
    if (i >= 12 && i < 38)
        return ('A' - 12 + i);
    if (i >= 38 && i < 63)
        return ('a' - 38 + i);
    return ('z');
}

// function to support encryption password for login
static char *crypt_make_salt(void) {
    time_t now;
    static unsigned long x;
    static char result[3];

    time(&now);
    x += now + getpid() + clock();
    result[0] = i64c(((x >> 18) ^ (x >> 6)) & 077);
    result[1] = i64c(((x >> 12) ^ x) & 077);
    result[2] = '\0';
    return result;
}

// function to support encryption password for login
static char *pw_encrypt(const char *clear, const char *salt) {
    static char cipher[128];
    char *cp;

#ifdef CONFIG_FEATURE_SHA1_PASSWORDS
    if (strncmp(salt, "$2$", 3) == 0) {
        return sha1_crypt(clear);
    }
#endif
    cp = (char *) crypt(clear, salt);
    /* if crypt (a nonstandard crypt) returns a string too large,
       truncate it so we don't overrun buffers and hope there is
       enough security in what's left */
    if (strlen(cp) > sizeof(cipher)-1) {
        cp[sizeof(cipher)-1] = 0;
    }
    strcpy(cipher, cp);
    return cipher;
}

/***************************************************************************
// Function Name: bcmCreateLoginCfg().
// Description  : create password file for login using 'admin' or 'support'.
// Parameters   : cp_admin - clear password of 'admin'.
//                cp_support - clear password of 'support'.
//                cp_user - clear password of 'user'.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmCreateLoginCfg(char *cp_admin, char *cp_support, char *cp_user) {
   struct passwd pw;
   FILE *fsPwd = NULL, *fsGrp = NULL;

   fsPwd = fopen("/etc/passwd", "w");

   if ( fsPwd != NULL ) {
      // In future, we may change uid of 'admin' and 'support'
      // uclibc may have a bug on putpwent in terms of uid,gid setup
      pw.pw_name = "admin";
      pw.pw_passwd = pw_encrypt(cp_admin, crypt_make_salt());
      pw.pw_uid = 0;
      pw.pw_gid = 0;
      pw.pw_gecos = "Administrator";
      pw.pw_dir = "/";
      pw.pw_shell = "/bin/sh";
      putpwent(&pw, fsPwd);

      pw.pw_name = "support";
      pw.pw_passwd = pw_encrypt(cp_support, crypt_make_salt());
      pw.pw_uid = 0;
      pw.pw_gid = 0;
      pw.pw_gecos = "Technical Support";
      pw.pw_dir = "/";
      pw.pw_shell = "/bin/sh";
      putpwent(&pw, fsPwd);

      pw.pw_name = "user";
      pw.pw_passwd = pw_encrypt(cp_user, crypt_make_salt());
      pw.pw_uid = 0;
      pw.pw_gid = 0;
      pw.pw_gecos = "Normal User";
      pw.pw_dir = "/";
      pw.pw_shell = "/bin/sh";
      putpwent(&pw, fsPwd);

      fclose(fsPwd);

      fsGrp = fopen("/etc/group", "w");
      if ( fsGrp != NULL ) {
         fprintf(fsGrp, "root::0:root,admin,support,user\n");
         fclose(fsGrp);
         return FILE_OPEN_OK;
      }
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmSetRemoteAccess().
// Description  : store remote access info to file.
// Parameters   : access - 1:enable, 0:disable.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmSetRemoteAccess(int access) {
   FILE* fs = fopen("/var/remoteaccess", "w");

   if ( fs != NULL ) {
      fprintf(fs, "%d\n", access);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmSetIpExtension().
// Description  : store PPP IP extension info to file.
// Parameters   : ipExt - 1:enable, 0:disable.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmSetIpExtension(int ipExt) {
   FILE* fs = fopen("/var/ipextension", "w");

   if ( fs != NULL ) {
      fprintf(fs, "%d\n", ipExt);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmGetIpExtension().
// Description  : retrieve PPP IP extension info from file.
// Parameters   : str -- buffer.
//                len -- size of buffer.
// Returns      : status 0 - OK, -1 - ERROR.
****************************************************************************/
int bcmGetIpExtension(char *str, int len) {
   FILE* fs = fopen("/var/ipextension", "r");

   if ( fs != NULL ) {
      fgets(str, len, fs);
      fclose(fs);
      return FILE_OPEN_OK;
   }

   return FILE_OPEN_ERR;
}

/***************************************************************************
// Function Name: bcmGetIfcNameByIndex().
// Description  : get interface name by its index.
// Parameters   : ifcIdx -- interface index.
//                ifcName -- interface name.
// Returns      : interface name or NULL
****************************************************************************/
char *bcmGetIfcNameByIndex(int ifcIdx, char *ifcName) {
   struct ifreq ifr;
   int s = 0;

   if ( ifcName == NULL ) return NULL;

   ifcName[0] = '\0';
   if ( (s = socket(AF_INET, SOCK_DGRAM, 0)) < 0 ) return NULL;

   ifr.ifr_ifindex = ifcIdx;
   if ( ioctl(s, SIOCGIFNAME, &ifr) < 0 ) {
      close(s);
      return NULL;
   }

   strcpy(ifcName, ifr.ifr_name);
   close(s);

   return ifcName;
}

/***************************************************************************
// Function Name: bcmIsValidIfcName.
// Description  : validate the interface name.
// Parameters   : ifcName -- interface name that need to validate.
// Returns      : TRUE or FALSE
****************************************************************************/
int bcmIsValidIfcName(char *ifcName) {
   int ret = FALSE, index = 1;
   char name[SYS_CMD_LEN];

   while ( bcmGetIfcNameByIndex(index, name) != NULL ) {
      if ( strcmp(ifcName, name) == 0 ) {
         ret = TRUE;
         break;
      }
      index++;
   }

   return ret;
}

/***************************************************************************
// Function Name: bcmGetIfcNameById.
// Description  : get interface name by its id (defined in ifcdefs.h).
// Parameters   : ifcId -- interface Id.
//                ifcName -- interface name.
// Returns      : interface name or NULL
****************************************************************************/
char *bcmGetIfcNameById(int ifcId, char *ifcName) {
   if ( ifcName == NULL ) return NULL;

   ifcName[0] = '\0';

   if ( ifcId >= IFC_ENET_ID && ifcId < IFC_USB_ID )
      sprintf(ifcName, "eth%d", ifcId - IFC_ENET_ID);
   else if ( ifcId >= IFC_USB_ID && ifcId < IFC_HPNA_ID )
//      strcpy(ifcName, "eth1");
      sprintf(ifcName, "usb%d", ifcId - IFC_USB_ID);
   else if ( ifcId >= IFC_HPNA_ID && ifcId < IFC_WIRELESS_ID )
      sprintf(ifcName, "il%d", ifcId - IFC_HPNA_ID);
   else if ( ifcId >= IFC_WIRELESS_ID && ifcId < IFC_WIRELESS_ID + IFC_LAN_MAX )
//      strcpy(ifcName, "eth2");
      sprintf(ifcName, "wl%d", ifcId - IFC_WIRELESS_ID);

   return ifcName;
}

/***************************************************************************
// Function Name: bcmSetConnTrackMax.
// Description  : tune the connection track table size.
// Parameters   : none.
// Returns      : none.
****************************************************************************/
void bcmSetConnTrackMax(void) {
   int conntrack_max = 1000;
   char cmd[SYS_CMD_LEN];
   FILE* fs = fopen("/proc/sys/net/ipv4/ip_conntrack_max", "r");

   // init ip_conntrack_max
   if ( fs != NULL ) {
      fgets(cmd, SYS_CMD_LEN, fs);
      conntrack_max = atoi(cmd);
      fclose(fs);
   }
   if ( conntrack_max < 1000 ) {
      conntrack_max *= 2;
      if ( conntrack_max > 1000 )
         conntrack_max = 1000;
   }
   sprintf(cmd, "echo \"%d\" > /proc/sys/net/ipv4/ip_conntrack_max", conntrack_max);
   bcmSystem(cmd);
}

/***************************************************************************
// Function Name: bcmResetConnTrackTable.
// Description  : reset the connection track table.
// Parameters   : none.
// Returns      : none.
****************************************************************************/
void bcmResetConnTrackTable(void) {
   if ( bcmIsModuleInserted("ip_conntrack") == TRUE )
      bcmSystem("echo > /proc/net/ip_conntrack");
}

/***************************************************************************
// Function Name: bcmHandleConnTrack.
// Description  : handle the connection track table.
// Parameters   : none.
// Returns      : none.
****************************************************************************/
void bcmHandleConnTrack(void) {
   bcmResetConnTrackTable();
   bcmSetConnTrackMax();
}

/***************************************************************************
// Function Name: bcmInsertModules.
// Description  : insert all modules under the given path.
// Parameters   : path -- the given path.
// Returns      : none.
****************************************************************************/
void bcmInsertModules(char *path) {
   struct dirent *entry;
   DIR *dir;
   char *cp = NULL;

   if ( path == NULL ) return;

   dir = opendir(path);
   if ( dir == NULL ) return;

   while ( (entry = readdir(dir)) != NULL )
      if ( (cp = strstr(entry->d_name, ".o")) != NULL ) {
         *cp = '\0';
         bcmInsertModule(entry->d_name);
      }
   closedir(dir);
}

/***************************************************************************
// Function Name: bcmInsertModule.
// Description  : insert module with the given name.
// Parameters   : modName -- the given module name.
// Returns      : none.
****************************************************************************/
void bcmInsertModule(char *modName) {
   char cmd[SYS_CMD_LEN];

   if ( bcmIsModuleInserted(modName) == FALSE ) {
      sprintf(cmd, "insmod %s", modName);
      bcmSystemMute(cmd);
   }
}

/***************************************************************************
// Function Name: bcmIsModuleInserted.
// Description  : verify the given module name is already inserted or not.
// Parameters   : modName -- the given module name.
// Returns      : TRUE or FALSE.
****************************************************************************/
int bcmIsModuleInserted(char *modName) {
   int ret = FALSE;
   char buf[SYS_CMD_LEN];
   FILE* fs = fopen("/proc/modules", "r");

   if ( fs != NULL ) {
      while ( fgets(buf, SYS_CMD_LEN, fs) > 0 )
         if ( strstr(buf, modName) != NULL ) {
            ret = TRUE;
            break;
         }
      fclose(fs);
   }

   return ret;
}

/***************************************************************************
// Function Name: bcmRemoveIpTableRule().
// Description  : remove IP table rules.
// Parameters   : device -- interface name.
//                table -- IP table name.
//                chain -- IP table chain.
// Returns      : 1 - Success. 0 - Fail
****************************************************************************/
int bcmRemoveIpTableRule(char *device, char *table, char *chain) {
   char col[9][IFC_SMALL_LEN];
   char col10[IFC_LARGE_LEN], line[IFC_GIANT_LEN];
   int ret = FALSE, count = 0;
   FILE* fs = NULL;

   // execute iptables command to create iptable file
   sprintf(line, "iptables -t %s -L %s -v --line-numbers > /var/iptable",
           table, chain);
   bcmSystemMute(line);

   fs = fopen("/var/iptable", "r");
   if ( fs != NULL ) {
      while ( fgets(line, sizeof(line), fs) ) {
         // read pass 2 header lines
         if ( count++ < 2 ) continue;
         sscanf(line, "%s %s %s %s %s %s %s %s %s %s %s",
                col[0], col[1], col[2], col[3], col[4], col[5],
                col[6], col[7], col[8], col[9], col10);
         // if chain rule for the given device is found
         if ( strcmp(col[6], device) == 0 ||
              strcmp(col[7], device) == 0 ) {
            sprintf(line, "iptables -t %s -D %s %s", table, chain, col[0]);
            bcmSystem(line);
            ret = TRUE;
            break;
         }
      }
      fclose(fs);
   }

   // remove iptable file
   bcmSystemMute("rm /var/iptable");

   return ret;
}

/***************************************************************************
// Function Name: bcmRemoveAllIpTableRules().
// Description  : remove all IP table rules attach with the given device.
// Parameters   : device -- interface name.
// Returns      : none
****************************************************************************/
void bcmRemoveAllIpTableRules(char *device) {
   while ( bcmRemoveIpTableRule(device, "filter", "INPUT") == TRUE )
      ;

   while ( bcmRemoveIpTableRule(device, "filter", "FORWARD") == TRUE )
      ;

   while ( bcmRemoveIpTableRule(device, "filter", "OUTPUT") == TRUE )
      ;

   while ( bcmRemoveIpTableRule(device, "nat", "PREROUTING") == TRUE )
      ;
/* We should keep the masquerading rules
   while ( bcmRemoveIpTableRule(device, "nat", "POSTROUTING") == TRUE )
      ;
*/

   while ( bcmRemoveIpTableRule(device, "nat", "OUTPUT") == TRUE )
      ;
}

/***************************************************************************
// Function Name: bcmRemoveRipIpTableRule().
// Description  : remove RIP IP table rules.
// Parameters   : none.
// Returns      : 1 - Success. 0 - Fail
****************************************************************************/
int bcmRemoveRipIpTableRule(void) {
   char col[9][IFC_SMALL_LEN];
   char col10[IFC_LARGE_LEN], line[IFC_GIANT_LEN];
   int ret = FALSE, count = 0;
   FILE* fs = NULL;

   // execute iptables command to create iptable file
   sprintf(line, "iptables -L INPUT -v --line-numbers > /var/iptable");
   bcmSystemMute(line);

   fs = fopen("/var/iptable", "r");
   if ( fs != NULL ) {
      while ( fgets(line, sizeof(line), fs) ) {
         // read pass 2 header lines
         if ( count++ < 2 ) continue;
         sscanf(line, "%s %s %s %s %s %s %s %s %s %s %s",
                col[0], col[1], col[2], col[3], col[4], col[5],
                col[6], col[7], col[8], col[9], col10);
         // if protocol column is "udp" and last colum is "udp dpt:route"
         // then delete rule since it is RIP IP tables rule
         if ( strcmp(col[4], "udp") == 0 ||
              strcmp(col10, "udp dpt:route") == 0 ) {
            sprintf(line, "iptables -D INPUT %s", col[0]);
            bcmSystemMute(line);
            ret = TRUE;
            break;
         }
      }
      fclose(fs);
   }

   // remove iptable file
   bcmSystemMute("rm /var/iptable");

   return ret;
}

/***************************************************************************
// Function Name: bcmGetDefaultRouteInterfaceName().
// Description  : get interface name that is used for first default route.
// Parameters   : ifcName -- the return interface name, '\0' if not found.
// Returns      : none
****************************************************************************/
void bcmGetDefaultRouteInterfaceName(char *ifcName) {
   char col[11][IFC_SMALL_LEN];
   char line[IFC_GIANT_LEN];
   int count = 0;

   if ( ifcName == NULL ) return;

   ifcName[0] = '\0';

   FILE* fsRoute = fopen("/proc/net/route", "r");
   if ( fsRoute != NULL ) {
      while ( fgets(line, sizeof(line), fsRoute) ) {
         // read pass header line
         if ( count++ < 1 ) continue;
         sscanf(line, "%s %s %s %s %s %s %s %s %s %s %s",
                col[0], col[1], col[2], col[3], col[4], col[5],
                col[6], col[7], col[8], col[9], col[10]);
         // if destination && mask are 0 then it's default route
         if ( strcmp(col[1], "00000000") == 0 &&
              strcmp(col[7], "00000000") == 0 ) {
            strcpy(ifcName, col[0]);
            break;
         }
      }
      fclose(fsRoute);
   }
}

//**************************************************************************
// Function Name: bcmIsInterfaceNatEnabled
// Description  : check the given interface is NAT enabled or not.
// Parameters   : ifcName -- the given interface name.
// Returns      : TRUE or FALSE.
//**************************************************************************
int bcmIsInterfaceNatEnabled(char *ifcName) {
   int ret = FALSE;
   UINT16 ifcId = 0, wanId = 0, retLen = 0;
   IFC_WAN_IFC_INFO infoWan;
   PSI_STATUS status = PSI_STS_OK;
   PSI_HANDLE psiWan = BcmPsi_appOpen(WAN_APPID);

   // ifcName should have the following format: pppxx, nasxx, atmxx
   // where x is number
   ifcId = atoi(&ifcName[3]);
   if ( ifcId <= IFC_BR2684_ID ) return ret;

   // bridge
   if ( ifcId > IFC_BR2684_ID && ifcId <= IFC_MER2684_ID )
      wanId = ifcId - IFC_BR2684_ID;
   // mer
   else if ( ifcId > IFC_MER2684_ID && ifcId <= IFC_PPPOE2684_ID )
      wanId = ifcId - IFC_MER2684_ID;
   // pppoe
   else if ( ifcId > IFC_PPPOE2684_ID && ifcId <= IFC_PPPOA2684_ID )
      wanId = ifcId - IFC_PPPOE2684_ID;
   // pppoa
   else if ( ifcId > IFC_PPPOA2684_ID && ifcId <= IFC_IPOA2684_ID )
      wanId = ifcId - IFC_PPPOA2684_ID;
   // ipoa
   else
      wanId = ifcId - IFC_IPOA2684_ID;

   status = BcmPsi_objRetrieve(psiWan, wanId, &infoWan,
                               sizeof(infoWan), &retLen);
   if ( status == PSI_STS_OK )
      ret = infoWan.enblNat;

   return ret;
}

/***************************************************************************
// Function Name: bcmGetIntfNameSocket.
// Description  : Return the interface name a socket is bound to
// Parameters   : socketfd: Socket descriptor, intfname: Network interface name
// Returns      : Failed: -1. Succeeded: 0
****************************************************************************/
int bcmGetIntfNameSocket(int socketfd, char *intfname)
{

   int i,fd;
   struct ifreq *all_ifr;
   struct ifconf ifc;
   struct sockaddr local_addr;
   int local_len;
   int numifs,bufsize;

   if (getsockname(socketfd, &local_addr,&local_len) < 0) {
     printf("bcmGetIntfNameSocket: Error in getsockname!\n");
     return -1;
   }

   //printf("bcmGetIntfNameSocket: Session comes from: %s\n",inet_ntoa(((struct sockaddr_in *)&local_addr)->sin_addr));
   
   if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
     printf("bcmGetIntfNameSocket: Error openning socket when getting socket intface info\n");
     return -1;
   }

   numifs = 16;

   bufsize = numifs*sizeof(struct ifreq);
   all_ifr = (struct ifreq *)malloc(bufsize);
   if (all_ifr == NULL) {
      printf("bcmGetIntfNameSocket: out of memory!\n");
      close(fd);
      return -1;
   }

   ifc.ifc_len = bufsize;
   ifc.ifc_buf = (char *)all_ifr;
   if (ioctl(fd, SIOCGIFCONF, &ifc) < 0) {
      printf("bcmGetIntfNameSocket: Error getting interfaces\n");
      close(fd);
      free(all_ifr);
      return -1;
   }

   numifs = ifc.ifc_len/sizeof(struct ifreq);
   //printf("bcmGetIntfNameSocket: numifs=%d\n",numifs);
   for (i = 0; i < numifs; i ++) {
	   //printf("bcmGetIntfNameSocket: intface name=%s\n",all_ifr[i].ifr_name); 
      usleep(50); // need this for getting the ifname from telnetd
	   struct in_addr addr1,addr2;
	   addr1 = ((struct sockaddr_in *)&(local_addr))->sin_addr;
	   addr2 = ((struct sockaddr_in *)&(all_ifr[i].ifr_addr))->sin_addr;
	   if (addr1.s_addr == addr2.s_addr) {
		strcpy(intfname, all_ifr[i].ifr_name);
	   	break;
	   }
   }

   close(fd);
   free(all_ifr);
   return 0;
}

//***************************************************************************
// Function Name: parseStrInfo
// Description  : parse info to get value of the given variable.
// Parameters   : info -- information string.
//                var -- variable string to be searched in info string.
//                val -- value string after variable string to be returned.
//                len -- size of value string.
// Returns      : none.
//***************************************************************************/
void parseStrInfo(char *info, char *var, char *val, int len) {
   char *pChar = NULL;
   int i = 0;

   if ( info == NULL || var == NULL || val == NULL ) return;

   pChar = strstr(info, var);
   if ( pChar == NULL ) return;

   // move pass the variable string in line
   pChar += strlen(var);

   // Remove spaces from beginning of value string
   while ( *pChar != '\0' && isspace((int)*pChar) != 0 )
      pChar++;

   // get data until end of line, or space char
   for ( i = 0;
         i < len && *pChar != '\0' &&
         isspace((int)*pChar) == 0;
         i++, pChar++ )
      val[i] = *pChar;

   val[i] = '\0';
}

//**************************************************************************
// Function Name: bcmConvertStrToShellStr
// Description  : convert the given string so that each its character is
//                surround by '. If character is ' then it is surround by ".
//                (a#b'c"d => 'a''#''b'"'"'c''"''d')
// Parameters   : str - the given string.
//                buf - the converted string.
// Returns      : none.
//**************************************************************************
void bcmConvertStrToShellStr(char *str, char *buf) {
   if ( buf == NULL ) return;

   int len = strlen(str);
   int i = 0, j = 0;

   for ( i = 0; i < len; i++ ) {
      if ( str[i] != '\'' ) {
         buf[j++] = '\'';
         buf[j++] = str[i];
         buf[j++] = '\'';
      } else {
         buf[j++] = '"';
         buf[j++] = str[i];
         buf[j++] = '"';
      }
   }

   buf[j]  = '\0';
}

//**************************************************************************
// Function Name: bcmProcessMarkStrChars
// Description  : use backslash in front one of the escape codes to process
//                marked string characters.
//                (a'b"c => a\'b\"c)
// Parameters   : str - the string that needs to process its special chars.
// Returns      : none.
//**************************************************************************
void bcmProcessMarkStrChars(char *str) {
   if ( str == NULL ) return;
   if ( str[0] == '\0' ) return;

   char buf[SYS_CMD_LEN];
   int len = strlen(str);
   int i = 0, j = 0;

   for ( i = 0; i < len; i++ ) {
      if ( bcmIsMarkStrChar(str[i]) == TRUE )
         buf[j++] = '\\';
      buf[j++] = str[i];
   }

   buf[j] = '\0';
   strcpy(str, buf);
}

//**************************************************************************
// Function Name: bcmIsMarkStrChar
// Description  : verify the given character is used to mark the begining or
//                ending of string or not.
// Parameters   : c -- the given character.
// Returns      : TRUE or FALSE.
//**************************************************************************
int bcmIsMarkStrChar(char c) {
   char specChars[] = { '\'', '"', '\\' };
   int len = strlen(specChars);
   int i = 0;
   int ret = FALSE;

   for ( i = 0; i < len; i++ )
      if ( c == specChars[i] )
         break;

   if ( i < len )
      ret = TRUE;

   return ret;
}

static char glbSwVer[SYS_CMD_LEN];

//**************************************************************************
// Function Name: bcmSetSwVer
// Description  : store software version to global variable
// Parameters   : swVer - the software version that needs to be set.
// Returns      : none.
//**************************************************************************
void bcmSetSwVer(char *swVer) {
   if ( swVer == NULL ) return;

   if ( strlen(swVer) < SYS_CMD_LEN - 1 )
      strcpy(glbSwVer, swVer);
   else {
      strncpy(glbSwVer, swVer, SYS_CMD_LEN - 2);
      glbSwVer[SYS_CMD_LEN - 1] = '\0';
   }
}

//**************************************************************************
// Function Name: bcmGetSwVer
// Description  : retrieve software version from global variable
// Parameters   : swVer - buffer to get the software version.
//                size - size of buffer.
// Returns      : none.
//**************************************************************************
void bcmGetSwVer(char *swVer, int size) {
   if ( swVer == NULL ) return;

   if ( strlen(glbSwVer) < size - 1 )
      strcpy(swVer, glbSwVer);
   else {
      strncpy(swVer, glbSwVer, size - 2);
      swVer[size - 1] = '\0';
   }
}

//**************************************************************************
// Function Name: bcmSetDDNS
// Description  : 
// Parameters   : 
//                
// Returns      : none.
//**************************************************************************
void bcmSetDDNS(void) {
#define HTTP_PORT 80

    FILE *fp = NULL;
    UINT16 retLen = 0;
    UINT16 i = 0;
    int sock;
    char TX_Buffer[512], temp_buf[128], b64User[IFC_LARGE_LEN], b64Buf[IFC_LARGE_LEN];
    char ddnsWanIpAddr[IFC_TINY_LEN], Server[IFC_MEDIUM_LEN];
    WEB_NTWK_VAR infoWan;
    PSI_STATUS status = PSI_STS_OK;
    IFC_DDNS_INFO ddnsInfo;
    PSI_HANDLE sysPsi = BcmPsi_appOpen(SYSTEM_APPID);

 
    struct sockaddr_in addr;
    struct hostent *hostinfo;


    /* Retrieve DDNS user parameters */
    status = BcmPsi_objRetrieve(sysPsi, SYS_DDNS_ID, &ddnsInfo, sizeof(ddnsInfo), &retLen);

    if (status != PSI_STS_OK)
    return; 

    /* Check if user started client */
    if (strcmp(ddnsInfo.State, "Start") != 0) 
    return; 

    /* Retrieve Wan IP Address from first active PVC */
    for ( i = 1; i <= IFC_WAN_MAX; i++ ) {
       if ( BcmIcm_webGetWanInfo(&infoWan, i) != PSI_STS_OK) continue;

    if ( infoWan.enblService != TRUE )
    return;
       
       if ( infoWan.ntwkPrtcl != PROTO_BRIDGE )
         BcmIcm_webGetWanIpAddress(ddnsWanIpAddr, IFC_TINY_LEN, infoWan.ntwkPrtcl, i );
       else return;  
     } 

    /* Remove unnecessary extra chars returned by BcmIcm_webGetWanIpAddress function */
    for (i=0; i<= 16; i++){
       if ((ddnsWanIpAddr[i] == 13) || (ddnsWanIpAddr[i] == 10))
       ddnsWanIpAddr[i] ='\0';
    }

    /* Check if Wan IP Address changed */
    if ( strcmp(ddnsWanIpAddr, ddnsInfo.WanIpAddr) == 0)
    return;

    /* Save new Wan IP Address */
    strcpy (ddnsInfo.WanIpAddr, ddnsWanIpAddr);
    BcmPsi_objStore(sysPsi, SYS_DDNS_ID, &ddnsInfo, sizeof(ddnsInfo));
    BcmPsi_flush();

    /* Compute base64 username:password */
    sprintf(b64Buf, "%s:%s",ddnsInfo.UserName, ddnsInfo.Password);
    base64enc(b64Buf, b64User, sizeof(b64User));


    if ( strcmp(ddnsInfo.ServiceName, "DynDNS.org") == 0){    

    /* Build DynDns HTTP Request */   
    strcpy(Server, "www.dyndns.org");
    snprintf(TX_Buffer, sizeof(TX_Buffer), "GET /nic/update?system=dyndns&hostname=%s&myip=%s HTTP/1.0\n"
                                           "Host: members.dyndns.org\n"
                                           "Authorization: Basic %s\n"
                                           "User-Agent: USR9006 V0.1 - lionel_barreiro@usr.com\n"
                                           "Content-type: text/html\n\n",
                                           ddnsInfo.HostName, ddnsInfo.WanIpAddr, b64User);
    }

    else if ( strcmp(ddnsInfo.ServiceName, "DtDNS") == 0){    

    /* Build DtDNS HTTP Request */  
    strcpy(Server, "www.dtdns.com");
    snprintf(TX_Buffer, sizeof(TX_Buffer), "GET /api/autodns.cfm?id=%s&pw=%s&ip=%s HTTP/1.0\n"
                                           "Host: www.dtdns.com\n"
                                           "User-Agent: USR9006 V0.1 - lionel_barreiro@usr.com\n\n",
                                           ddnsInfo.HostName, ddnsInfo.Password, ddnsInfo.WanIpAddr);
    } 

    else if ( strcmp(ddnsInfo.ServiceName, "No-IP.com") == 0){    

    /* Build No-IP HTTP Request */   
    strcpy(Server, "www.no-ip.com");
    snprintf(TX_Buffer, sizeof(TX_Buffer), "GET /update.php?username=%s&pass=%s&host=%s&ip=%s HTTP/1.0\n"
                                           "Host: dynupdate.no-ip.com\n"
                                           "Connection: close\n"
					   "User-Agent: USR9006 V0.1 - lionel_barreiro@usr.com\n\n",
                                            ddnsInfo.UserName, ddnsInfo.Password, ddnsInfo.HostName, ddnsInfo.WanIpAddr);
    } else return;

				

    printf("\nSend HTTP request:\n%s", TX_Buffer);

    if(!(hostinfo = gethostbyname(Server))) {
       fprintf(stderr, "Unable to resolve host - %s\n", Server);
       return;
    }

    if((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
       fprintf(stderr, "Unable to get network socket\n");
       return;
    }

    addr.sin_family = AF_INET;
    addr.sin_port = htons(HTTP_PORT);
    addr.sin_addr = *(struct in_addr*)*hostinfo->h_addr_list;

    if(connect(sock, (struct sockaddr*)&addr, sizeof(addr)) == -1) {
       fprintf(stderr, "Unable to connect\n");
       close(sock);
       return;
    }

    /* send the request to server */
    write(sock, TX_Buffer, strlen(TX_Buffer));

      printf("Waiting for response....\n\n");

    /* get server reply */
    fp = fdopen(sock, "r");

    while(!feof(fp))
       fgets(temp_buf, sizeof(temp_buf), fp);

    fclose(fp);
    close(sock);

    /* Last received line in temp contains the return status message */
    printf("Return message: %s\n", temp_buf);
    cgiSetVar("ddnsUpdateStatus", temp_buf);
    return;
}


/*  FROM wget.c !!!
 *  Base64-encode character string
 *  oops... isn't something similar in uuencode.c?
 *  It would be better to use already existing code
 */
char *base64enc(char *p, char *buf, int len) {

        char al[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
                    "0123456789+/";
		char *s = buf;

        while(*p) {
				if (s >= buf+len-4)
					printf("Buffer Overflow");
                *(s++) = al[(*p >> 2) & 0x3F];
                *(s++) = al[((*p << 4) & 0x30) | ((*(p+1) >> 4) & 0x0F)];
                *s = *(s+1) = '=';
                *(s+2) = 0;
                if (! *(++p)) break;
                *(s++) = al[((*p << 2) & 0x3C) | ((*(p+1) >> 6) & 0x03)];
                if (! *(++p)) break;
                *(s++) = al[*(p++) & 0x3F];
        }

		return buf;
}
